#!/usr/bin/env bash

# The repetition of tile types in the "tiles" string weights repeated tiles to appear more often
# Default width and height are 100 if not otherwise specified on the command line
# zeroPad is 4 positions but can be changed to 3 for older versions of FreeCiv, also rename the (river) e03_ layer heading to "n"

declare    temp tile tiles="fggghppdjms" zeroPad="0000"
declare -a terrainLayer riverLayer
declare -i i j width=${1:-100} height=${2:-100} x y qty xMove yMove

while [ "${1}" != "" ]; do
	case "${1}" in
		-x | -X | -w | -W | --width		)	shift ; width="${1}" ;;
		-y | -Y | -h | -H | --height		)	shift ; height="${1}" ;;
		* )		echo "Usage: ${0} -x 50 -y 50" ; exit 1 ;;
	esac
	shift
done

if [ ${width} -lt 16 -o ${height} -lt 16 ] ; then
	echo "FreeCiv minimum map width/height is 16"
	exit 1
fi

# Generate terrain layer array filled with spaces (shallow ocean)
terrainLayer[0]=$(printf "%*s" ${width})
riverLayer[0]=$(printf -v temp %*s ${width} ; printf '%s\n' "${temp// /0}")
for ((i=1;i<${height};i++)) ; do
	terrainLayer[${i}]=${terrainLayer[0]}
	riverLayer[${i}]=${riverLayer[0]}
done

# coordinates of ~center tile
x=$((${width}/2))
y=$((${height}/2))


# Change the first/middle(ish) tile
tile="${tiles:$(( ${RANDOM} % ${#tiles} )):1}"
terrainLayer[${y}]="${terrainLayer[$((${y}-1))]:0:$((${x}-1))}${tile}${terrainLayer[${y}]:${x}}"

# Next move is 2 downward from last tile (counter-clockwise spiral)
qty=2
xMove=0
yMove=1


for ((j=1;j<=${qty};j++)) ; do
	for ((i=1;i<=${qty};i++)) ; do
		tile="${tiles:$(( ${RANDOM} % ${#tiles} )):1}"
		x=$((${x}+${xMove}))
		y=$((${y}+${yMove}))
		[ ${x} -lt 0 -o ${x} -ge ${width} ] && break 2
		[ ${y} -lt 0 -o ${y} -ge ${height} ] && break 2
		terrainLayer[${y}]="${terrainLayer[${y}]:0:$((${x}-1))}${tile}${terrainLayer[${y}]:${x}}"

		# 1 in 10 chance of river on tile which is 10% of the land tiles
		if [ $(( ${RANDOM} % 10 )) -eq 0 ] ; then
			riverLayer[${y}]="${riverLayer[${y}]:0:$((${x}-1))}2${riverLayer[${y}]:${x}}"
		fi
	done

	if [ ${xMove} -eq 0 ] ; then
		# If last move was down, then right is next (counter-clockwise)
		if [ ${yMove} -eq 1 ] ; then
			yMove=0 && xMove=1
		else # if last move was up, then left is next
			yMove=0 && xMove=-1
		fi
	elif [ ${xMove} -eq 1 ] ; then
			xMove=0 && yMove=-1	
		else
			xMove=0 && yMove=1
	fi

	qty=$((${qty}+1))
done

# Output terrain layer
for ((i=0;i<${height};i++)) ; do
	echo "t${zeroPad:${#i}}${i}=\"${terrainLayer[${i}]}\""
done

# Output river layer
echo
for ((i=0;i<${height};i++)) ; do
	echo "e03_${zeroPad:${#i}}${i}=\"${riverLayer[${i}]}\""
done

# Start positions aren't necessary, the game's selections work with Spiral Sea, unlike Wrecktangular Labyrinth

# Unresolved: pad the map edges for two ocean tiles all around the edges of the map?

exit 0
